/* Error handler.  */

#include "error.h"

#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

/* The calling program should define program_name and set it to the
   name of the executing program.  */
extern char *program_name;

/* Print the program name and error message MESSAGE.
   If ERRNUM is nonzero, print corresponding system error message.
   Exit with status STATUS if it is nonzero.  */
void
error (int status, int errnum, const char *message, ...)
{
     va_list args;

     fflush(stdout);
     fprintf(stderr, "%s: ", program_name);

     va_start(args, message);
     vfprintf(stderr, message, args);
     va_end(args);

     if (errnum) {
          char const *s;

          s = strerror(errnum);
          fprintf(stderr, ": %s", s);
     }
     putc('\n', stderr);
     fflush(stderr);
     if (status)
          exit(status);
}
