/* System-dependent declarations and other common stuff.  */

#ifndef _SYSTEM_H
#define _SYSTEM_H 1

/* Assume ANSI C89 headers are available.  */
#include <locale.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>

/* POSIX headers.  */
#include <errno.h>
#include <getopt.h>
#include <stdbool.h>
#include <unistd.h>

/* Names for the values of the `has_arg' field of `struct option'.  */
#define no_argument	        0
#define required_argument	1
#define optional_argument	2

/* Common --help and --version processing stuff.  */

/* These enum values cannot possibly conflict with the option values
   ordinarily used by commands, including CHAR_MAX + 1, etc.  Avoid
   CHAR_MIN - 1, as it may equal -1, the getopt end-of-options value.  */
enum
{
     GETOPT_HELP_CHAR = (CHAR_MIN - 2),
     GETOPT_VERSION_CHAR = (CHAR_MIN - 3)
};

#define GETOPT_HELP_OPTION_DECL \
     "help", no_argument, NULL, GETOPT_HELP_CHAR
#define GETOPT_VERSION_OPTION_DECL \
     "version", no_argument, NULL, GETOPT_VERSION_CHAR

#define case_GETOPT_HELP_CHAR			\
  case GETOPT_HELP_CHAR:			\
    usage(EXIT_SUCCESS);			\
    break;

#define case_GETOPT_VERSION_CHAR(Program_name, Version)                 \
     case GETOPT_VERSION_CHAR:						\
     printf("%s %s\n.", Program_name, Version);                         \
     exit (EXIT_SUCCESS);                                               \
     break;

#define HELP_OPTION_DESCRIPTION \
     "      --help     display this help and exit\n"
#define VERSION_OPTION_DESCRIPTION \
     "      --version  output version information and exit\n"

/* Set program_name, based on argv[0].  */
#ifndef SET_PROGRAM_NAME
# define SET_PROGRAM_NAME(argv0)                                \
     do {                                                       \
          const char *slash;                                    \
                                                                \
          slash = strrchr((argv0), '/');                        \
          program_name = (slash != NULL ? slash + 1 : (argv0)); \
     } while(0)
#endif

#define ISDIGIT(c) ((unsigned int) (c) - '0' <= 9)

#ifndef MAX
# define MAX(a, b) ((a) > (b) ? (a) : (b))
#endif

#ifndef MIN
# define MIN(a,b) (((a) < (b)) ? (a) : (b))
#endif

#ifndef ARRAY_CARDINALITY
# define ARRAY_CARDINALITY(Array) (sizeof (Array) / sizeof *(Array))
#endif

#endif /* system.h */
